@@ -1,10 +1,6 @@
-# $Id: Alias.pm 42214 2014-09-11 11:10:43Z wsl $
+# $Id: Alias.pm 46806 2017-07-26 15:48:10Z wsl $
 # $URL: https://svn.uvt.nl/its-id/trunk/sources/kiki/lib/UvT/Kiki/Database/Alias.pm $
 
-use utf8;
-use strict;
-use warnings FATAL => 'all';
-
 package UvT::Kiki::Database::Alias;
 
 use Email::Address;
@@ -231,6 +227,46 @@ sub store {
 	}
 }
 
+# A textual representation of how this entry is different from an older version.
+# If the older version is undef, a pseudo-diff is generated.
+sub diff {
+	my $full = $self->full;
+	my $ab = $self->addressbook;
+	my @destinations = map { $_->full } @{$self->destinations};
+	my $diff = $ab ? "$full (in address book):\n" : "$full: (not in address book)\n";
+
+	if(my $old = $self->original) {
+		$ab = $old->addressbook
+			? $ab
+				? "in address book"
+				: "no longer in address book"
+			: $ab
+				? "now in address book"
+				: "not in address book";
+
+		$diff = "$full ($ab):\n";
+		
+		my %destinations; @destinations{@destinations} = ();
+		my @old_destinations = map { $_->full } @{$old->destinations};
+		my %old_destinations; @old_destinations{@old_destinations} = ();
+		my %all_destinations = (%destinations, %old_destinations);
+
+		foreach my $destination (sort { lc($a) cmp lc($b) || $a cmp $b } keys %all_destinations) {
+			$diff .= exists $destinations{$destination}
+				? exists $old_destinations{$destination}
+					? " \t$destination\n"
+					: "+\t$destination\n"
+				: "-\t$destination\n";
+		}
+	} else {
+		foreach my $destination (sort { lc($a) cmp lc($b) || $a cmp $b } @destinations) {
+			$diff .= " \t$destination\n";
+		}
+	}
+
+	return $diff;
+}
+
 sub toString {
 	return $self->full . ": "
 		. join(', ', sort { lc($a) cmp lc($b) } map { $_->full } @{$self->destinations})
